@@ -45,7 +45,6 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KProtocolManager>
-#include <KUrlMimeData>
 
 #include <QAbstractItemView>
 #include <QActionGroup>
@@ -758,14 +757,12 @@ void DolphinView::cutSelectedItemsToClipboard()
 {
     QMimeData* mimeData = selectionMimeData();
     KIO::setClipboardDataCut(mimeData, true);
-    KUrlMimeData::exportUrlsToPortal(mimeData);
     QApplication::clipboard()->setMimeData(mimeData);
 }
 
 void DolphinView::copySelectedItemsToClipboard()
 {
-    QMimeData *mimeData = selectionMimeData();
-    KUrlMimeData::exportUrlsToPortal(mimeData);
+    QMimeData* mimeData = selectionMimeData();
     QApplication::clipboard()->setMimeData(mimeData);
 }
 
@@ -1345,7 +1342,7 @@ void DolphinView::slotItemCreated(const QUrl& url)
 
 void DolphinView::slotJobResult(KJob *job)
 {
-    if (job->error()) {
+    if (job->error() && job->error() != KIO::ERR_USER_CANCELED) {
         Q_EMIT errorMessage(job->errorString());
     }
     if (!m_selectedUrls.isEmpty()) {
